from math import pi
from constants import *
#
# define shapes and transformation
#
# color(color,part)
# circle(x0, y0, r)
# cylinder(x0, y0, z0, z1, r)
# cone(x0, y0, z0, z1, r0)
# sphere(x0, y0, z0, r)
# torus(x0, y0, z0, r0, r1)
# rectangle(x0, x1, y0, y1)
# cube(x0, x1, y0, y1, z0, z1)
# triangle(x0, y0, x1, y1, x2, y2) (points in clockwise order)
# pyramid(x0, x1, y0, y1, z0, z1)
# function(Z_of_XY)
# functions(upper_Z_of_XY,lower_Z_of_XY)
# add(part1, part2)
# subtract(part1, part2)
# intersect(part1, part2)
# move(part,dx,dy)
# translate(part,dx,dy,dz)
# rotate(part, angle)
# rotate_x(part, angle)
# rotate_y(part, angle)
# rotate_z(part, angle)
# rotate_90(part)
# rotate_180(part)
# rotate_270(part)
# reflect_x(part,x0)
# reflect_y(part,y0)
# reflect_z(part,z0)
# reflect_xy(part)
# reflect_xz(part)
# reflect_yz(part)
# scale_x(part, x0, sx)
# scale_y(part, y0, sy)
# scale_z(part, z0, sz)
# scale_xy(part, x0, y0, sxy)
# scale_xyz(part, x0, y0, z0, sxyz)
# coscale_x_y(part, x0, y0, y1, angle0, angle1, amplitude, offset)
# coscale_x_z(part, x0, z0, z1, angle0, angle1, amplitude, offset)
# coscale_xy_z(part, x0, y0, z0, z1, angle0, angle1, amplitude, offset)
# taper_x_y(part, x0, y0, y1, s0, s1)
# taper_x_z(part, x0, z0, z1, s0, s1)
# taper_xy_z(part, x0, y0, z0, z1, s0, s1)
# shear_x_y(part, y0, y1, dx0, dx1)
# shear_x_z(part, z0, z1, dx0, dx1)

True = "1"
False = "0"

def circle(x0, y0, r):
   return "(((X-({0}))*(X-({0})) + (Y-({1}))*(Y-({1}))) <= ({2}*{2}))".format(x0, y0, r)

def rectangle(x0, x1, y0, y1):
   return "((X >= ({0})) & (X <= ({1})) & (Y >= ({2})) & (Y <= ({3})))".format(x0, x1, y0, y1)

def right_triangle(x0, y0, l):
   return "((X > {0}) & (X < {0} + {2} - (Y-{1})) & {1})".format(x0, y0, l)
   
def triangle(x0, y0, x1, y1, x2, y2): # points in clockwise order
   return ("((((({y1})-({y0}))*(X-({x0}))-(({x1})-({x0}))*(Y-({y0}))) >= 0) & " + \
           "(((({y2})-({y1}))*(X-({x1}))-(({x2})-({x1}))*(Y-({y1}))) >= 0) & " + \
           "(((({y0})-({y2}))*(X-({x2}))-(({x0})-({x2}))*(Y-({y2}))) >= 0))").format(x0 = x0,
                                                                                     y0 = y0,
                                                                                     x1 = x1,
                                                                                     y1 = y1,
                                                                                     x2 = x2,
                                                                                     y2 = y2)
def tab(x, y, width = tab_width, squish = tab_squish, height = material_thickness, chamfer = 0.2, rotation = 0):
    width += squish / 2
    tab = rectangle(0, width / 2, 0, height)
    tab = subtract(tab, triangle(width / 2 - chamfer*height, height,
                             width / 2, height,
                             width / 2, height * (1 - chamfer)))
    tab = add(tab, reflect_x(tab, 0))
    if rotation:
        tab = rotate(tab, rotation)
    tab = move(tab, x, y)
    return tab

def slot(x, y, width = tab_width, height = material_thickness, chamfer = 0.2, rotation = 0):
    slot = rectangle(0, width/2, -height, 0)
    slot = add(slot, triangle(width/2, 0,
                              width/2 + height * chamfer, 0,
                              width/2, -chamfer*height))
    slot = add(slot, reflect_x(slot, 0))
    if rotation:
        slot = rotate(slot, rotation)
    slot = move(slot, x, y)
    return slot

def add(part1, part2):
   return "{0} | {1}".format(part1, part2)

def subtract(part1, part2):
   return "({0}) & ~({1})".format(part1, part2)

def intersect(part1, part2):
   return "({0}) & ({1})".format(part1, part2)

def move(part,dx,dy):
   part = part.replace('X','(X-('+str(dx)+'))')
   part = part.replace('Y','(Y-('+str(dy)+'))')
   return part   

def rotate(part, angle):
   angle = str(angle*pi/180)
   part = part.replace('X','(cos({0})*X+sin({0})*y)'.format(angle))
   part = part.replace('Y','(-sin({0})*X+cos({0})*y)'.format(angle))
   part = part.replace('y','Y')
   return part

def rotate_90(part):
   part = reflect_y(part,0)
   part = reflect_xy(part)
   return part

def rotate_180(part):
   part = rotate_90(part)
   part = rotate_90(part)
   return part

def rotate_270(part):
   part = rotate_90(part)
   part = rotate_90(part)
   part = rotate_90(part)
   return part

def reflect_x(part, x0 = 0):
   return part.replace('X','({0!s}-X)'.format(x0))

def reflect_y(part, y0 = 0):
   return part.replace('Y','({0!s}-Y)'.format(y0))

def reflect_xy(part):
   part = part.replace('X','temp')
   part = part.replace('Y','X')
   part = part.replace('temp','Y')
   return part